﻿using UnityEngine;
using System.Collections;
using System;
using System.Linq;
using System.Runtime.InteropServices;

public class tower2rtcmix : MonoBehaviour {
	int objno = 2; // change this for multiple RTcmix instantiations
	float[] vals = new float[1024]; // for MAXMESSAGE values
	char[] printvals = new char[8192]; // for printing RTcmix messages/errors

	private float Rval;

	void Start () {
		int status;

		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		status = DLL.RTcmix_init(objno);

		// set it for single-precision float samples (see RTcmix_API.h), scaled between -1.0 and 1.0, 2 chans;
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		DLL.RTcmix_setAudioBufferFormat(8, 2, objno);

		// note the "1024" here, vs. "2048" in the Unity code
		// this is the difference between 'frames' and 'nsamples'
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		status = DLL.RTcmix_setparams(44100, 2, 1024, 1, 32, objno); // '32' is the nmber of RTcmix busses

		String score = "x1 = 0.5 " +
			"ampenv1 = maketable(\"line\", 1000, 0,0, 1,1, 10,0) " +
			"indexenv = maketable(\"line\", 1000, 0,0.1, 1,1, 2,0) " +
			"wave = maketable(\"wave\", 1000, \"sine\") " +
			"notes = { 7.00, 7.02, 7.04, 7.06, 7.08, 7.10, 8.00, 8.02, 8.04, " +
			"	8.06, 8.08, 9.00 } " +
			"lnotes = len(notes) " +
			"MAXBANG(0) ";
		SendScore(score);
	}


	// Update is called once per frame
	void Update () {
		GameObject rolleything = GameObject.Find ("rolleything");
		Transform rolleytransform = rolleything.transform;

		float distance = Vector3.Distance (rolleytransform.position, transform.position);

		// distance is between 65 and 4.5, scale to between 1.0 - 3.999
		Rval = (float)(3.999 - (3.0 * (distance-4.5)/60.0));
	}


	void OnApplicationQuit(){
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		DLL.RTcmix_destroy (objno); // reset RTcmix
	}


	// this function is called by unity for every buffer of sound samples computed
	void OnAudioFilterRead(float[] data, int channels){
		unsafe{
			fixed (float* b = data) {
				while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
				// null input, synthesis only.  If you were processing sound, the (void *)null
				// would be replaced by (void *)b
				// Also, 1024 instead of 2048 because it is frames, not nsamples
				DLL.RTcmix_runAudio((void *)null, (void *)b, 1024, objno);


				// uncomment this to process MAXBANGs
				if (DLL.unity_checkForBang(objno) == 1) {
					String score = "R = " + Rval + "  " +
						"if (R > 3.999) R = 3.99 " +
						"ampmult1 = (R-1.0)/3.0 " +
						"if (R > 1.0) " +
						"	x1 = R * x1 * (1.0 - x1) " +
						"index = trunc(x1 * lnotes) " +
						"note = notes[index] " +
						"freq = cpspch(note) " +
						"if (ampmult1 > 0) " +
						"	FMINST(0, 0.2, 20000*ampenv1*ampmult1, freq, 2*freq, 1, 3, random(), wave, indexenv) " +
						"MAXBANG(0.1) ";
					SendScore(score);			
				}


				// uncomment this to process MAXMESSAGEs
				/*
				fixed (float *vptr = vals) {
					// this shows how to access the values from a MAXMESSAGE
					int nvals = DLL.unity_checkForVals(vptr, objno);
					if (nvals > 0) {
						Debug.Log("got some vals, " + nvals + " and vals[1] is " + vals[1]);
					}
				}
				*/


				// uncomment this to print RTcmix messages/errors in the system console
				/*
				fixed (char *pptr = printvals) {
					int pcheck = DLL.unity_checkForPrint(pptr, objno);
					while (pcheck > 0) {
						string s = new string(printvals);
						Debug.Log(s);
						Array.Clear(printvals, 0, printvals.Length);
						pcheck = DLL.unity_checkForPrint(pptr, objno);
					}
				}
				*/
			}
		}
	}


	// this is the score-sending function
	public void SendScore(String score) {
		IntPtr sptr = Marshal.StringToHGlobalAnsi(score);
		unsafe {
			while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
			DLL.unity_parse_score ((char*)sptr.ToPointer (), score.Length, objno);
		}
		Marshal.FreeHGlobal(sptr);
	}
}

