﻿using UnityEngine;
using System.Collections;
using System;
using System.Linq;
using System.Runtime.InteropServices;

public class tower3rtcmix : MonoBehaviour {
	int objno = 3; // change this for multiple RTcmix instantiations
	float[] vals = new float[1024]; // for MAXMESSAGE values
	char[] printvals = new char[8192]; // for printing RTcmix messages/errors

	void Start () {
		int status;

		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		status = DLL.RTcmix_init(objno);

		// set it for single-precision float samples (see RTcmix_API.h), scaled between -1.0 and 1.0, 2 chans;
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		DLL.RTcmix_setAudioBufferFormat(8, 2, objno);

		// note the "1024" here, vs. "2048" in the Unity code
		// this is the difference between 'frames' and 'nsamples'
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		status = DLL.RTcmix_setparams(44100, 2, 1024, 1, 32, objno); // '32' is the nmber of RTcmix busses

		String score = "bus_config(\"WAVETABLE\", \"aux 0-1 out\") " +
			"bus_config(\"MOOGVCF\", \"aux 0-1 in\", \"out 0-1\") " +
			"ampfreq = makeconnection(\"inlet\", 1, 50) " +
			"wave = maketable(\"wave\", 1000, \"saw\") " +
			"WAVETABLE(0, 9999, 20000, 5.00, 0.5, wave) " +
			"WAVETABLE(0, 9999, 20000, 5.001, 0.5, wave) " +
			"WAVETABLE(0, 9999, 20000, 4.1187, 0.5, wave) " +
			"MOOGVCF(0, 0, 9999, ampfreq/100, 0, 0.5, 0, ampfreq*100, 0.1) ";
		SendScore(score);
	}


	// Update is called once per frame
	void Update () {
		GameObject rolleything = GameObject.Find ("rolleything");
		Transform rolleytransform = rolleything.transform;

		float distance = Vector3.Distance (rolleytransform.position, transform.position);

		// distance is between 65 and 4.5, scale between 0 - 100
		float pval = (float)(1.0 - (distance-4.5)/60.0);
		pval = pval * 100.0f;

		if (pval > 0.0) {
			while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
			DLL.RTcmix_setPField (1, pval, objno);
		}
	}


	void OnApplicationQuit(){
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		DLL.RTcmix_destroy (objno); // reset RTcmix
	}


	// this function is called by unity for every buffer of sound samples computed
	void OnAudioFilterRead(float[] data, int channels){
		unsafe{
			fixed (float* b = data) {
				while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
				// null input, synthesis only.  If you were processing sound, the (void *)null
				// would be replaced by (void *)b
				// Also, 1024 instead of 2048 because it is frames, not nsamples
				DLL.RTcmix_runAudio((void *)null, (void *)b, 1024, objno);

				// uncomment this to process MAXMESSAGEs
				/*
				fixed (float *vptr = vals) {
					// this shows how to access the values from a MAXMESSAGE
					int nvals = DLL.unity_checkForVals(vptr, objno);
					if (nvals > 0) {
						Debug.Log("got some vals, " + nvals + " and vals[1] is " + vals[1]);
					}
				}
				*/


				// uncomment this to print RTcmix messages/errors in the system console
				/*
				fixed (char *pptr = printvals) {
					int pcheck = DLL.unity_checkForPrint(pptr, objno);
					while (pcheck > 0) {
						string s = new string(printvals);
						Debug.Log(s);
						Array.Clear(printvals, 0, printvals.Length);
						pcheck = DLL.unity_checkForPrint(pptr, objno);
					}
				}
				*/
			}
		}
	}


	// this is the score-sending function
	public void SendScore(String score) {
		IntPtr sptr = Marshal.StringToHGlobalAnsi(score);
		unsafe {
			while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
			DLL.unity_parse_score ((char*)sptr.ToPointer (), score.Length, objno);
		}
		Marshal.FreeHGlobal(sptr);
	}
}

